loadAPI(1);

/* Script Initilization */

/* MPD226 MIDI Product ID */
const PRODUCT_ID = 0x35;
//const minimumPadOff    = 882;
const minimumPadOff    = 930;
const minimumPadOn     = 994;

var noteInputs = [];

load("MPD2_common.js");
load("MPD2_PadInstrument.js");
load("MPD2_PadClipLaunch.js");
load("MPD2_PadSceneLaunch.js");

host.defineController("Akai", "MPD226", "1.0.2", "71ecc229-b531-48cc-9c54-e4fa304ea449");
host.defineMidiPorts(2, 2);


if (host.platformIsWindows()) {
    host.addDeviceNameBasedDiscoveryPair(["MPD226","MIDIIN4 (MPD226)"], ["MPD226","MIDIOUT4 (MPD226)"]);
}
else if (host.platformIsMac()) {
    host.addDeviceNameBasedDiscoveryPair(["MPD226 Port A","MPD226 Remote"], ["MPD226 Port A","MPD226 Remote"]);
}
else {
    host.addDeviceNameBasedDiscoveryPair(["MPD226 MIDI 1","MPD226 MIDI 4"], ["MPD226 MIDI 1","MPD226 MIDI 4"]);
}

function init()
{
    PadNotes = host.getMidiInPort(0).createNoteInput("MPD226 Pads", "DF????","EF??", "AF????");
    PadNotes.setKeyTranslationTable(PadMIDITable.ON);
    PadNotes.setShouldConsumeEvents(false);
    
    
    initClipArray();
    //noteInputs[10] = PadNotes;
    
    transport = host.createTransport();
    application = host.createApplication();
    trackBank = host.createMainTrackBank(8, 2, 16);
    sceneLaunchTrackBank = host.createTrackBank(1,0,64);
    
    cursorTrack = host.createArrangerCursorTrack(2, 0);
    cursorDevice = host.createEditorCursorDevice();
    
    primaryDevice = cursorTrack. createCursorDevice("Primary");
    primaryDevice.addNameObserver(11, "", cursorTrackInstrumentNameObs());
    
    cursorTrack.addColorObserver(cursorTrackColorObs());
    cursorTrack.addPitchNamesObserver(cursorTrackpitchObs());
    host.getMidiInPort(0).setMidiCallback(onMidi);
    
    
    trackBank.getClipLauncherScenes().addNameObserver(sceneLaunchObs());
    
    for (var p = 0; p < 8; p++)
    {
        var macro = primaryDevice.getMacro(p).getAmount();
        var parameter = cursorDevice.getParameter(p);
        var track = trackBank.getTrack(p);
        
        track.getArm().addValueObserver(armObsFunction(p));
        track.getMute().addValueObserver(muteObsFunction(p));
        track.getSolo().addValueObserver(soloObsFunction(p));
        
        macro.setIndication(true);
        parameter.setIndication(true);
        parameter.setLabel("P" + (p + 1));
        track.getVolume().setIndication(true);
        track.getPan().setIndication(true);
        track.getSend(0).setIndication(true);
        track.getSend(1).setIndication(true);
        
        var clipLauncherSlots = track.getClipLauncherSlots();
        clipLauncherSlots.addIsPlayingObserver(clipPlayingObs(p));
        clipLauncherSlots.addHasContentObserver(clipContentObs(p));
        clipLauncherSlots.addIsRecordingObserver(clipRecordObs(p));
        clipLauncherSlots.setIndication(true);
    }
    setActivePadMode(PadInstrument);
    

    println("Akai Profressional MPD226 Bitwig Controller Script");
}


